# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:\Users\Reinh\OneDrive\Dokumente\python\MzChess\configureEngine.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(634, 301)
        Dialog.setSizeGripEnabled(True)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Dialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.cancelPushButton = QtWidgets.QPushButton(Dialog)
        self.cancelPushButton.setMinimumSize(QtCore.QSize(75, 0))
        self.cancelPushButton.setMaximumSize(QtCore.QSize(75, 16777215))
        self.cancelPushButton.setObjectName("cancelPushButton")
        self.gridLayout.addWidget(self.cancelPushButton, 4, 4, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.removePushButton = QtWidgets.QPushButton(Dialog)
        self.removePushButton.setMinimumSize(QtCore.QSize(75, 0))
        self.removePushButton.setMaximumSize(QtCore.QSize(75, 16777215))
        self.removePushButton.setObjectName("removePushButton")
        self.gridLayout.addWidget(self.removePushButton, 4, 1, 1, 1)
        self.browsePushButton = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.browsePushButton.sizePolicy().hasHeightForWidth())
        self.browsePushButton.setSizePolicy(sizePolicy)
        self.browsePushButton.setMinimumSize(QtCore.QSize(23, 0))
        self.browsePushButton.setMaximumSize(QtCore.QSize(23, 16777215))
        self.browsePushButton.setText("")
        self.browsePushButton.setObjectName("browsePushButton")
        self.gridLayout.addWidget(self.browsePushButton, 1, 6, 1, 1)
        self.nameLineEdit = QtWidgets.QLineEdit(Dialog)
        self.nameLineEdit.setMinimumSize(QtCore.QSize(0, 23))
        self.nameLineEdit.setObjectName("nameLineEdit")
        self.gridLayout.addWidget(self.nameLineEdit, 0, 1, 1, 5)
        self.engineTableWidget = QtWidgets.QTableWidget(Dialog)
        self.engineTableWidget.setEditTriggers(QtWidgets.QAbstractItemView.CurrentChanged|QtWidgets.QAbstractItemView.SelectedClicked)
        self.engineTableWidget.setWordWrap(False)
        self.engineTableWidget.setColumnCount(2)
        self.engineTableWidget.setObjectName("engineTableWidget")
        self.engineTableWidget.setRowCount(0)
        self.engineTableWidget.horizontalHeader().setVisible(False)
        self.gridLayout.addWidget(self.engineTableWidget, 3, 0, 1, 6)
        self.savePushButton = QtWidgets.QPushButton(Dialog)
        self.savePushButton.setMinimumSize(QtCore.QSize(75, 0))
        self.savePushButton.setMaximumSize(QtCore.QSize(75, 16777215))
        self.savePushButton.setObjectName("savePushButton")
        self.gridLayout.addWidget(self.savePushButton, 4, 0, 1, 1)
        self.detailsPushButton = QtWidgets.QPushButton(Dialog)
        self.detailsPushButton.setObjectName("detailsPushButton")
        self.gridLayout.addWidget(self.detailsPushButton, 1, 5, 1, 1)
        self.okPushButton = QtWidgets.QPushButton(Dialog)
        self.okPushButton.setMinimumSize(QtCore.QSize(75, 0))
        self.okPushButton.setMaximumSize(QtCore.QSize(75, 16777215))
        self.okPushButton.setObjectName("okPushButton")
        self.gridLayout.addWidget(self.okPushButton, 4, 5, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 4, 2, 1, 2)
        self.executableLineEdit = QtWidgets.QLineEdit(Dialog)
        self.executableLineEdit.setObjectName("executableLineEdit")
        self.gridLayout.addWidget(self.executableLineEdit, 1, 1, 1, 4)
        self.horizontalLayout.addLayout(self.gridLayout)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Configure Engine"))
        self.label_2.setText(_translate("Dialog", "Executable:"))
        self.cancelPushButton.setText(_translate("Dialog", "Cancel"))
        self.label.setText(_translate("Dialog", "Name:"))
        self.removePushButton.setText(_translate("Dialog", "Remove"))
        self.savePushButton.setText(_translate("Dialog", "Save"))
        self.detailsPushButton.setText(_translate("Dialog", "Details"))
        self.okPushButton.setText(_translate("Dialog", "OK"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())
